@@ -26,6 +26,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	<fcntl.h>
 #include	<limits.h>
 #include	<errno.h>
+#include	<openssl/sha.h>
 #include	"cdb.h"
 #include	"cdb_make.h"
 #include	"cdb_seq.h"
@@ -37,6 +38,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	"missing_p.h"
 #include	"elcerror_p.h"
 #include	"xml_p.h"
+#include	"dbinfo.h"
+#include	"show.h"
 #ifdef		ELC_FIND_LEAKS
 #include	"leakfind.h"
 #endif
@@ -53,17 +56,36 @@ static void print_missing_xml(FILE *out, options *opts,
     putc('\n', out);
 }
 
-static void print_missing_abbrev(FILE *out, options *opts,
-			      const char *key, size_t klen)
+static void print_missing_lines(FILE *out, options *opts,
+				const char *key, size_t klen,
+				const dbinfo *dbinf, size_t vlen)
 {
-    fputs("missing: ", stdout);
+    const struct stat	*s	 = &dbinf->stat;
+
+    fputs("missing: ", out);
     while (klen--)
       putchar(*key++);
+    fputs("   ", out);
+
+    /* show some interesting information about the missing file: pugzm */
+    if (S_ISLNK(s->st_mode)) /* for a symlink ... */
+      fputs("p(sym) ", out); /* display "sym" instead of "777" */
+    else
+      show_octal(out, 'p', s->st_mode & PERM_MASK);
+    
+    show_long(out, 'u', s->st_uid);
+    show_long(out, 'g', s->st_gid);
+    show_long(out, 'z', s->st_size);
+    show_time(out, 'm', s->st_mtime);
+    
+    if (vlen == sizeof(dbinfo)) /* if there's a checksum */
+      show_checksum(out, dbinf->sum, sizeof(dbinf->sum));
     putchar('\n');
 }
 
 static void do_currdb_check(options *opts, struct cdb *curr_cdb,
-			    char *key, size_t klen)
+			    char *key, size_t klen,
+			    dbinfo *dbinf, size_t vlen)
 {
     int		ret;
 
@@ -75,7 +97,7 @@ static void do_currdb_check(options *opts, struct cdb *curr_cdb,
       if (opts->output == OUTPUT_XML)
 	print_missing_xml(stdout, opts, key, klen);
       else
-	print_missing_abbrev(stdout, opts, key, klen);
+	print_missing_lines(stdout, opts, key, klen, dbinf, vlen);
     }
 }
 
@@ -97,6 +119,7 @@ void check_for_missing(options *opts)
     int		err;
     unsigned	ksiz	 = 1024; /* beginning key space allotment */
     char	*key	 = malloc(ksiz);
+    dbinfo	dbinf;
     size_t	klen, vlen;
 
     if (! key)
@@ -117,15 +140,15 @@ void check_for_missing(options *opts)
       else if (err == 1)	/* no more in db sequence */
 	break;
       while (klen > ksiz) {
-	if (ksiz > (UINT_MAX / 2))
+	if (ksiz > (UINT_MAX / 2)) /* guard against overflow */
 	  DIE("key size too big");
 	ksiz	 *= 2;
 	if (! (key = realloc(key, ksiz)) )
 	  DIE("realloc key");
       }
-      if (cdb_seq_getkey(&knownseq, key, klen, vlen) == -1)
+      if (cdb_seq_get(&knownseq, key, klen, &dbinf, vlen) == -1)
 	DIE("cdb_seq_getkey");
-      do_currdb_check(opts, &curr_cdb, key, klen);
+      do_currdb_check(opts, &curr_cdb, key, klen, &dbinf, vlen);
     }
 
     free(key);
