@@ -44,7 +44,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define	EATSPACE(buf)	do {				\
     while (*(buf)					\
 	   && (*(buf) != '\n')				\
-	   && (isspace(*(buf))))	/* skip whitespace */	\
+	   && (isspace((int) *(buf))))	/* skip whitespace */	\
       ++(buf);						\
 } while (0)    
 
@@ -120,7 +120,9 @@ void options_init(options *o)
 {
     o->conffile		 = NULL;
     o->knowndbname	 = NULL;
+    memset(&o->knowndb, 0, sizeof(o->knowndb));
     o->currdbname	 = NULL;
+    memset(&o->currdb, 0, sizeof(o->currdb));
     o->root		 = NULL;
     if (! (o->ruleset = malloc(sizeof(hashtbl_t))) )
       DIE("malloc hashtbl");
@@ -243,11 +245,11 @@ inline static void do_rule(options *o, char *buf)
 	die(__FUNCTION__, "Error: filename too long in config");
 
       if (*buf == '\\') {
-	if (isspace(*(buf + 1)))
+	if (isspace((int) *(buf + 1)))
 	  namebuf[namebuf_chars++]	 = *(++buf);
 	else
 	  namebuf[namebuf_chars++]	 = '\\';
-      } else if (isspace(*buf)) {
+      } else if (isspace((int) *buf)) {
 	break;			/* end of the name */
       } else {
 	namebuf[namebuf_chars++]	 = *buf;
@@ -258,7 +260,7 @@ inline static void do_rule(options *o, char *buf)
     EATSPACE(buf);
     n_switches	 = strlen(buf);
     /* trim trailing whitespace */
-    while (n_switches && isspace(buf[n_switches - 1])) {
+    while (n_switches && isspace((int) buf[n_switches - 1])) {
       buf[n_switches - 1]	 = '\0';
       --n_switches;
     }
